import java.util.Hashtable;
import java.io.*;

import javax.naming.*;
import javax.naming.directory.*;

/**
 * Demonstrates how to search the directory while automatically
 * following referrals, but set a limit on the chain of referrals followed.
 *
 * usage: java Throw
 */
class Throw {
    public static void main(String[] args) throws IOException {

	// Set up environment for creating initial context
	Hashtable env = new Hashtable(11);
	env.put(Context.INITIAL_CONTEXT_FACTORY, 
	    "com.sun.jndi.ldap.LdapCtxFactory");
	env.put(Context.PROVIDER_URL, "ldap://localhost:489/o=JNDITutorial");

	// Set referral property to throw ReferralException
	env.put(Context.REFERRAL, "throw");

	try {
	    // Create initial context
	    DirContext ctx = new InitialDirContext(env);

	    // Set controls for performing subtree search
	    SearchControls ctls = new SearchControls();
	    ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);

	    while (true) {
		try {
		    // Perform search
		    NamingEnumeration answer = ctx.search("", "(objectclass=*)", 
			ctls);

		    // Print the answer
		    while (answer.hasMore()) {
			System.out.println(">>>" + ((SearchResult)answer.next()).getName());
		    }
		    break;  // search completes with no more referrals

		} catch (ReferralException e) {

		    if (! followReferral(e.getReferralInfo())) {
			if (! e.skipReferral()) {
			    break;  // no more referrals to proceess
			}
		    }
		    // point to the new context
		    ctx = (DirContext) e.getReferralContext();
		}
	    }
	} catch (NamingException e) {
	    e.printStackTrace();
	}
    }

    /**
      * Displays referral information and asks user whether to follow.
      */
    private static boolean followReferral(Object referralInfo) throws IOException {
	byte[] reply = new byte[4];

	System.out.println("Received referral: " + referralInfo);
	System.out.print("Follow it? [y/n]");
	System.in.read(reply);

	return (reply[0] != 'n');
    }
}

